@@ -95,6 +95,8 @@ public:
     crush = crush_create();
     assert(crush);
     have_rmaps = false;
+
+    set_tunables_default();
   }
 
   // tunables
@@ -104,20 +106,21 @@ public:
     crush->choose_total_tries = 19;
     crush->chooseleaf_descend_once = 0;
   }
-  void set_tunables_optimal() {
+  void set_tunables_argonaut() {
+    set_tunables_legacy();
+  }
+  void set_tunables_bobtail() {
     crush->choose_local_tries = 0;
     crush->choose_local_fallback_tries = 0;
     crush->choose_total_tries = 50;
     crush->chooseleaf_descend_once = 1;
   }
-  void set_tunables_argonaut() {
-    set_tunables_legacy();
-  }
-  void set_tunables_bobtail() {
-    set_tunables_optimal();
+
+  void set_tunables_optimal() {
+    set_tunables_bobtail();
   }
   void set_tunables_default() {
-    set_tunables_legacy();
+    set_tunables_bobtail();
   }
 
   int get_choose_local_tries() const {
@@ -158,6 +161,7 @@ public:
     return
       crush->chooseleaf_descend_once != 0;
   }
+  bool has_v2_rules() const;
 
   // bucket types
   int get_num_type_names() const {
@@ -337,7 +341,7 @@ public:
    * insert an item into the map at a specific position
    *
    * Add an item as a specific location of the hierarchy.
-   * Specifically, we look for the most specific location constriant
+   * Specifically, we look for the most specific location constraint
    * for which a bucket already exists, and then create intervening
    * buckets beneath that in order to place the item.
    *
@@ -563,6 +567,12 @@ public:
   int set_rule_step_take(unsigned ruleno, unsigned step, int val) {
     return set_rule_step(ruleno, step, CRUSH_RULE_TAKE, val, 0);
   }
+  int set_rule_step_set_choose_tries(unsigned ruleno, unsigned step, int val) {
+    return set_rule_step(ruleno, step, CRUSH_RULE_SET_CHOOSE_TRIES, val, 0);
+  }
+  int set_rule_step_set_chooseleaf_tries(unsigned ruleno, unsigned step, int val) {
+    return set_rule_step(ruleno, step, CRUSH_RULE_SET_CHOOSELEAF_TRIES, val, 0);
+  }
   int set_rule_step_choose_firstn(unsigned ruleno, unsigned step, int val, int type) {
     return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSE_FIRSTN, val, type);
   }
@@ -570,17 +580,17 @@ public:
     return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSE_INDEP, val, type);
   }
   int set_rule_step_choose_leaf_firstn(unsigned ruleno, unsigned step, int val, int type) {
-    return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSE_LEAF_FIRSTN, val, type);
+    return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSELEAF_FIRSTN, val, type);
   }
   int set_rule_step_choose_leaf_indep(unsigned ruleno, unsigned step, int val, int type) {
-    return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSE_LEAF_INDEP, val, type);
+    return set_rule_step(ruleno, step, CRUSH_RULE_CHOOSELEAF_INDEP, val, type);
   }
   int set_rule_step_emit(unsigned ruleno, unsigned step) {
     return set_rule_step(ruleno, step, CRUSH_RULE_EMIT, 0, 0);
   }
 
   int add_simple_rule(string name, string root_name, string failure_domain_type,
-		      ostream *err = 0);
+		      string mode, ostream *err = 0);
 
   int remove_rule(int ruleno);
 
@@ -620,7 +630,7 @@ private:
     if (!crush)
       return (-EINVAL);
 
-    if (item > 0)
+    if (item >= 0)
       return (-EINVAL);
 
     // check that the bucket that we want to detach exists
@@ -721,10 +731,10 @@ public:
 
   /* modifiers */
   int add_bucket(int bucketno, int alg, int hash, int type, int size,
-		 int *items, int *weights) {
+		 int *items, int *weights, int *idout) {
     crush_bucket *b = crush_make_bucket(alg, hash, type, size, items, weights);
     assert(b);
-    return crush_add_bucket(crush, bucketno, b);
+    return crush_add_bucket(crush, bucketno, b, idout);
   }
   
   void finalize() {
@@ -764,7 +774,8 @@ public:
 	       const vector<__u32>& weight) const {
     Mutex::Locker l(mapper_lock);
     int rawout[maxout];
-    int numrep = crush_do_rule(crush, rule, x, rawout, maxout, &weight[0], weight.size());
+    int scratch[maxout * 3];
+    int numrep = crush_do_rule(crush, rule, x, rawout, maxout, &weight[0], weight.size(), scratch);
     if (numrep < 0)
       numrep = 0;
     out.resize(numrep);
@@ -793,10 +804,13 @@ public:
   void dump(Formatter *f) const;
   void dump_rules(Formatter *f) const;
   void list_rules(Formatter *f) const;
+  void dump_tree(const vector<__u32>& w, ostream *out, Formatter *f) const;
   static void generate_test_instances(list<CrushWrapper*>& o);
 
 
   static bool is_valid_crush_name(const string& s);
+  static bool is_valid_crush_loc(CephContext *cct,
+				 const map<string,string> loc);
 };
 WRITE_CLASS_ENCODER(CrushWrapper)
 
