@@ -101,6 +101,9 @@ namespace librbd {
       utime_t elapsed;
       assert(lock.is_locked());
       elapsed = ceph_clock_now(ictx->cct) - start_time;
+      if (complete_cb) {
+	complete_cb(rbd_comp, complete_arg);
+      }
       switch (aio_type) {
       case AIO_TYPE_READ:
 	ictx->perfcounter->tinc(l_librbd_aio_rd_latency, elapsed); break;
@@ -114,9 +117,6 @@ namespace librbd {
 	lderr(ictx->cct) << "completed invalid aio_type: " << aio_type << dendl;
 	break;
       }
-      if (complete_cb) {
-	complete_cb(rbd_comp, complete_arg);
-      }
       done = true;
       cond.Signal();
     }
@@ -196,12 +196,10 @@ namespace librbd {
 
   class C_CacheRead : public Context {
   public:
-    C_CacheRead(Context *completion, AioRead *req)
-      : m_completion(completion), m_req(req) {}
+    explicit C_CacheRead(AioRead *req) : m_req(req) {}
     virtual ~C_CacheRead() {}
     virtual void finish(int r);
   private:
-    Context *m_completion;
     AioRead *m_req;
   };
 }
