@@ -590,7 +590,6 @@ bool MDSMonitor::preprocess_command(MMonCommand *m)
     }
     r = 0;
   } else if (prefix == "mds dump") {
-    string val;
     int64_t epocharg;
     epoch_t epoch;
 
@@ -653,7 +652,6 @@ bool MDSMonitor::preprocess_command(MMonCommand *m)
   } else if (prefix == "mds tell") {
     string whostr;
     cmd_getval(g_ceph_context, cmdmap, "who", whostr);
-    string args;
     vector<string>args_vec;
     cmd_getval(g_ceph_context, cmdmap, "args", args_vec);
 
