@@ -165,6 +165,12 @@ public:
     Incremental(bufferlist::iterator &p) {
       decode(p);
     }
+
+    pg_pool_t *get_new_pool(int64_t pool, const pg_pool_t *orig) {
+      if (new_pools.count(pool) == 0)
+	new_pools[pool] = *orig;
+      return &new_pools[pool];
+    }
   };
   
 private:
@@ -226,26 +232,6 @@ private:
     memset(&fsid, 0, sizeof(fsid));
   }
 
-  // no copying
-  /* oh, how i long for c++11...
-private:
-  OSDMap(const OSDMap& other) = default;
-  const OSDMap& operator=(const OSDMap& other) = default;
-public:
-  */
-
-  void deepish_copy_from(const OSDMap& o) {
-    *this = o;
-    pg_temp.reset(new map<pg_t,vector<int> >(*o.pg_temp));
-    osd_uuid.reset(new vector<uuid_d>(*o.osd_uuid));
-
-    // NOTE: this still references shared entity_addr_t's.
-    osd_addrs.reset(new addrs_s(*o.osd_addrs));
-
-    // NOTE: we do not copy crush.  note that apply_incremental will
-    // allocate a new CrushWrapper, though.
-  }
-
   // map info
   const uuid_d& get_fsid() const { return fsid; }
   void set_fsid(uuid_d& f) { fsid = f; }
@@ -273,14 +259,14 @@ public:
   int get_max_osd() const { return max_osd; }
   void set_max_osd(int m);
 
-  int get_num_osds() const {
+  unsigned get_num_osds() const {
     return num_osd;
   }
   int calc_num_osds();
 
   void get_all_osds(set<int32_t>& ls) const;
-  int get_num_up_osds() const;
-  int get_num_in_osds() const;
+  unsigned get_num_up_osds() const;
+  unsigned get_num_in_osds() const;
 
   int get_flags() const { return flags; }
   int test_flag(int f) const { return flags & f; }
@@ -503,7 +489,7 @@ public:
 private:
   /// pg -> (raw osd list)
   int _pg_to_osds(const pg_pool_t& pool, pg_t pg, vector<int>& osds) const;
-  void _remove_nonexistent_osds(vector<int>& osds) const;
+  void _remove_nonexistent_osds(const pg_pool_t& pool, vector<int>& osds) const;
 
   /// pg -> (up osd list)
   void _raw_to_up_osds(pg_t pg, vector<int>& raw, vector<int>& up) const;
@@ -610,10 +596,12 @@ public:
   /*
    * handy helpers to build simple maps...
    */
+  /// build crush bucket types.  @return 'root' type id
   void build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
 		    int num_osd, int pg_bits, int pgp_bits);
   int build_simple_from_conf(CephContext *cct, epoch_t e, uuid_d &fsid,
 			     int pg_bits, int pgp_bits);
+  static int _build_crush_types(CrushWrapper& crush);
   static void build_simple_crush_map(CephContext *cct, CrushWrapper& crush,
 				     map<int, const char*>& poolsets, int num_osd);
   static void build_simple_crush_map_from_conf(CephContext *cct, CrushWrapper& crush,
@@ -626,6 +614,7 @@ private:
 public:
   void print(ostream& out) const;
   void print_summary(Formatter *f, ostream& out) const;
+  void print_oneline_summary(ostream& out) const;
   void print_tree(ostream *out, Formatter *f) const;
 
   string get_flag_string() const;
@@ -641,7 +630,7 @@ WRITE_CLASS_ENCODER_FEATURES(OSDMap::Incremental)
 typedef std::tr1::shared_ptr<const OSDMap> OSDMapRef;
 
 inline ostream& operator<<(ostream& out, const OSDMap& m) {
-  m.print_summary(NULL, out);
+  m.print_oneline_summary(out);
   return out;
 }
 
