@@ -2,6 +2,7 @@
 
 export PYTHONPATH=./pybind
 export LD_LIBRARY_PATH=.libs
+export DYLD_LIBRARY_PATH=$LD_LIBRARY_PATH
 
 
 # abort on failure
@@ -237,6 +238,7 @@ fi
 $SUDO rm -f core*
 
 test -d out || mkdir out
+test -d dev || mkdir dev
 $SUDO rm -rf out/*
 test -d gmon && $SUDO rm -rf gmon/*
 
@@ -339,6 +341,7 @@ $DAEMONOPTS
 $COSDDEBUG
 $extra_conf
 [mon]
+        mon pg warn min per osd = 10
 $DAEMONOPTS
 $CMONDEBUG
 $extra_conf
@@ -389,7 +392,7 @@ EOF
 		    cmd="rm -rf $CEPH_DEV_DIR/mon.$f"
 		    echo $cmd
 		    $cmd
-                    cmd="mkdir $CEPH_DEV_DIR/mon.$f"
+                    cmd="mkdir -p $CEPH_DEV_DIR/mon.$f"
                     echo $cmd
                     $cmd
 		    cmd="$CEPH_BIN/ceph-mon --mkfs -c $conf -i $f --monmap=$monmap_fn"
